package cn.iocoder.yudao.module.member.mq.consumer.score.core;

import cn.iocoder.yudao.framework.mybatis.core.query.LambdaQuery;
import cn.iocoder.yudao.module.member.dal.dataobject.scoreRule.ScoreRuleDO;
import cn.iocoder.yudao.module.member.enums.ScoreRuleStatusEnum;
import cn.iocoder.yudao.module.member.enums.ScoreRuleTypeEnum;
import cn.iocoder.yudao.module.member.enums.TransportTypeEnum;
import cn.iocoder.yudao.module.member.service.scoreRule.ScoreRuleService;
import org.springframework.beans.factory.annotation.Autowired;

import java.util.Date;

/**
 * 半成品
 * @author zhangfeng
 */
public abstract class AbstractScoreRuleStrategy implements ScoreRuleStrategy {
    private static ScoreRuleDO orderVShipScoreRule = null;
    private static ScoreRuleDO orderVFlyScoreRule = null;
    private static ScoreRuleDO registerScoreRule = null;
    private static ScoreRuleDO recommendScoreRule = null;

    protected ScoreRuleService scoreRuleService;

    @Autowired
    public AbstractScoreRuleStrategy(ScoreRuleService scoreRuleService) {
        this.scoreRuleService = scoreRuleService;
    }

    public ScoreRuleDO getEnableScoreRuleByType(ScoreRuleTypeEnum scoreRuleType) {
        ScoreRuleDO scoreRuleDO = scoreRuleService.selectOne(new LambdaQuery<ScoreRuleDO>()
                .eq(ScoreRuleDO::getType, scoreRuleType.getValue())
                .eq(ScoreRuleDO::getStatus, ScoreRuleStatusEnum.ENABLED.getValue()));
        if (scoreRuleDO != null) {
            // 校验一下活动开始结束时间
            if (!scoreRuleDO.getEndTime().after(new Date()) || !scoreRuleDO.getStartTime().before(new Date())) {
                return null;
            } else {
                recommendScoreRule = scoreRuleDO;
                return recommendScoreRule;
            }
        }
        return null;
    }
    public ScoreRuleDO getEnableScoreRuleByType(ScoreRuleTypeEnum scoreRuleType, TransportTypeEnum transportType) {
        ScoreRuleDO scoreRuleDO = null;
        switch (scoreRuleType) {
            case ORDER_V:
                scoreRuleDO = scoreRuleService.getEnabledOrderVScoreRuleByTransportType(transportType.getValue());
                if (scoreRuleDO != null) {
                    if (!scoreRuleDO.getEndTime().after(new Date()) || !scoreRuleDO.getStartTime().before(new Date())) {
                        return null;
                    }
                    if (transportType == TransportTypeEnum.OCEAN_LCL) {
                        orderVShipScoreRule = scoreRuleDO;
                        return orderVShipScoreRule;
                    }
                    if (transportType == TransportTypeEnum.SPECIAL_LINE_AIR_FREIGHT) {
                        orderVFlyScoreRule = scoreRuleDO;
                        return orderVFlyScoreRule;
                    }

                }
                break;
            case RECOMMEND:
            case REGISTER:
                return getEnableScoreRuleByType(scoreRuleType);
        }
        return null;
    }


    @Override
    public void refreshScoreRule(ScoreRuleTypeEnum scoreRuleType) {
        switch (scoreRuleType) {
            case ORDER_V:
                orderVShipScoreRule = null;
                orderVFlyScoreRule = null;
                break;
            case RECOMMEND:
                recommendScoreRule = null;
                break;
            case REGISTER:
                registerScoreRule = null;
                break;
        }
    }
}

