package cn.iocoder.yudao.module.member.mq.consumer.score.core;

import cn.iocoder.yudao.framework.mybatis.core.query.LambdaQuery;
import cn.iocoder.yudao.module.member.api.score.MemberUserScoreApi;
import cn.iocoder.yudao.module.member.api.score.dto.MemberUserScoreOperateReqDTO;
import cn.iocoder.yudao.module.member.dal.dataobject.scoreRule.ScoreRuleDO;
import cn.iocoder.yudao.module.member.dal.dataobject.user.MemberUserDO;
import cn.iocoder.yudao.module.member.enums.ScoreRuleStatusEnum;
import cn.iocoder.yudao.module.member.enums.ScoreRuleTypeEnum;
import cn.iocoder.yudao.module.member.enums.ScoreSourceTypeEnum;
import cn.iocoder.yudao.module.member.mq.message.ScoreMessage;
import cn.iocoder.yudao.module.member.service.scoreRule.ScoreRuleService;
import cn.iocoder.yudao.module.member.service.user.MemberUserService;
import lombok.extern.slf4j.Slf4j;
import org.springframework.stereotype.Service;

import javax.annotation.Resource;
import java.util.Date;
import java.util.HashMap;

/**
 * @author zhangfeng
 */
@Service
@Slf4j
public class RegisterStrategy extends AbstractScoreRuleStrategy {
    @Resource
    private MemberUserScoreApi memberUserScoreApi;
    @Resource
    private MemberUserService memberUserService;

    public RegisterStrategy(ScoreRuleService scoreRuleService) {
        super(scoreRuleService);
    }

    @Override
    public void addScore(ScoreMessage message) {
        Long userId = message.getUserId();
        log.info("Received register message,userID:{}", userId);
        ScoreRuleDO scoreRuleDO = scoreRuleService.selectOne(new LambdaQuery<ScoreRuleDO>()
                .eq(ScoreRuleDO::getType, ScoreRuleTypeEnum.REGISTER.getValue())
                .eq(ScoreRuleDO::getStatus, ScoreRuleStatusEnum.ENABLED.getValue()));
        if (scoreRuleDO == null) {
            log.info("Register rule is not exist,userID:{}", userId);
            return;
        }
        // 是否过期，或者活动还没开始
        if (!scoreRuleDO.getEndTime().after(new Date()) || !scoreRuleDO.getStartTime().before(new Date())) {
            log.info("Register listener: The score rule has expired,scoreRuleID:{}", scoreRuleDO.getId());
            return;
        }
        MemberUserDO memberUserDO = memberUserService.getUser(userId);
        if (memberUserDO == null) {
            log.info("Register listener: The user does not exist,userID:{}", userId);
            return;
        }
        //TODO：校验注册入口，目前表没加
        try {
            HashMap<String, Object> map = new HashMap<>();
            map.put("userId", userId);
            map.put("userNameZh", message.getUserNameZh());
            map.put("userNameEn", message.getUserNameEn());
            String desensitizedPhoneNumber = message.getPhone().replaceAll("(\\d{3})\\d{4}(\\d{4})", "$1****$2");
            map.put("phone", desensitizedPhoneNumber);

            memberUserScoreApi.operateScore(MemberUserScoreOperateReqDTO.builder()
                    .memberId(userId)
                    .scoreCount(scoreRuleDO.getGetScoreOnce())
                    .sourceType(ScoreSourceTypeEnum.REGISTER)
                    .ruleId(scoreRuleDO.getId())
                    .uniqueId(String.valueOf(userId))
                    .expireDays(scoreRuleDO.getScorePeriod())
                    .build());
        } catch (Exception e) {
            log.error("Register listener: operateScore error", e);
        }
        log.info("Register rule add score success,ruleID:{},userID:{}", scoreRuleDO.getId(), userId);
    }

    @Override
    public ScoreRuleTypeEnum getStrategyScoreRuleType() {
        return ScoreRuleTypeEnum.REGISTER;
    }
}
